# +++++++++++++++++++++++++++++++++++++++++
# Interface R  CaliFloPP
# AB: 22-09-2009
# +++++++++++++++++++++++++++++++++++++++++
decodpoly <- function(X,  ficp)  {
  # FUNCTION
  # ecrit  un couple de polys dont on veut l'analyse,  sur
  # le fichier des paramtres
  # INPUT
  # X: vecteur des 2 polys voulus
  # ficp: nom du fichier o on crit les paramtres ou
  # connexion vers celui-ci
  # RETURN
  # rien
  # CALLED BY
  # decodparam,
  # (via un apply sur la matrice des couples de polys voulus)
  # -------------------------------------------
cat(X[1], X[2], "\n", file=ficp, append=TRUE)
invisible()
} # fin decodpoly


# -----------------------------------------------
decodparam <- function(X, nom, ficp)  {
  # FUNCTION
  # ecrit  un paramtre et sa valeur sur le fichier des paramtres
  # INPUT
  # X: valeur du paramtre
  # nom: nom du paramtre
  # ficp: nom du fichier o on crit les paramtres ou
  # connexion vers celui-ci
    # RETURN
  # rien
  # CALLED BY
  # - lui-mme, via un mapply sur la liste des paramtres
  # spcifiques   cub ou sur la liste des paramtres
  # spcifiques  grid
  # - califlopp, via un mapply sur la liste des paramtres
  # NOTE
  # delim="\t" (tabulation) ne marche pas
  # (faut-il utiliser l'argu allowEscapes de scan?? Mais dans caliconfig.h
  # le delim par dfaut est \t, donc inutile)
  # -------------------------------------------

switch(nom,
       nfunc  = {cat("nfunc", X, "\n", file=ficp, append=TRUE)},
       func  = {cat(X, "\n", file=ficp, append=TRUE)},
         input = {cat("input", X, "\n", file=ficp, append=TRUE)},
       output= {cat("output", X, "\n", file=ficp, append=TRUE)},
         verbose = {cat("verbose ", file=ficp, append=TRUE)
                    if (X==TRUE) cat("1",  file=ficp, append=TRUE) else
                    cat("0", file=ficp, append=TRUE)
                    cat("\n", file=ficp, append=TRUE)},
       delim= {cat("delim ",  file=ficp, append=TRUE);
#               cat(gsub("\"", "", deparse(X)),  file=ficp, append=TRUE);
               cat( deparse(X),  file=ficp, append=TRUE);
               cat("\n", file=ficp, append=TRUE)
               },
       poly = {
    # Decoder les couples de polys voulus
  if (is.list(X)) 
    X <- matrix(unlist(X), ncol=2, byrow=TRUE)
  if (is.vector(X)) {
    if (length(X) !=2)
      stop("Vector of polygons should be 2")
    X <- matrix(X, ncol=2)
  }
    cat("ncouples ", nrow(X), "\n", file=ficp, append=TRUE)
  apply(X, 1, decodpoly, ficp)
}, # fin poly
       method = { 
                    cat("method ", file=ficp, append=TRUE)
                   if (X=="grid") cat("1", file=ficp, append=TRUE) else cat("0",file= ficp, append=TRUE)
                    cat("\n", file=ficp, append=TRUE)},
       dz  = {  cat("dz ", X, "\n", file=ficp, append=TRUE) },
       dp  = {  cat("dp ", X, "\n", file=ficp, append=TRUE) },
       tz  = {  cat("tz ", X, "\n", file=ficp, append=TRUE) },
# cubature parameters       
         cub = { if (!is.list(X))
                 stop("Component cub of param should be a list")
                 mapply( decodparam, X, names(X), ficp)},
         maxpts = {  for (i in 1:length(X)) {
           if (!is.na(X[i])) { cat("maxpts ",i, "\n", file=ficp, append=TRUE)
                           cat(format(X[i], scientific=F),
                               "\n", file=ficp, append=TRUE) }
             } # fin for
                  },
       reler  = {  for (i in 1:length(X)) {
           if (!is.na(X[i])) { cat("reler ",i, "\n", file=ficp, append=TRUE)
                           cat(X[i], "\n", file=ficp, append=TRUE) }
             } # fin for
                  },
       abser  = {  for (i in 1:length(X)) {
           if (!is.na(X[i])) { cat("abser ",i, "\n", file=ficp, append=TRUE)
                           cat(X[i], "\n", file=ficp, append=TRUE) }
             } # fin for
                  },
# grid parameters       
         grid = { if (!is.list(X))
                 stop("Component grid of param should be a list")
                 mapply( decodparam, X, names(X), ficp)},
       seed =  {cat("seed", X, "\n", file=ficp, append=TRUE)},
       step =  {cat("stepx", X[1], "\nstepy", X[2],  "\n",
                    file=ficp, append=TRUE)},
       nr =  {cat("nr", X, "\n", file=ficp, append=TRUE)},

         stop(paste("Invalid name for a parameter:", nom))
         ) # fin switch
  invisible()
} # fin decodparam

# -----------------------------------------------        
califlopp <- function(file, dispf=c(1,2), param=NULL, resfile=NULL)
{
  # FUNCTION
  # Appelle califlopp_sd (mode non-interactif)
  # (via un programme C wrapper)
  # INPUT
  # file: pathname of the polygons file
  # dispf: the R dispersion functions
  # integer vector for the compiled functions, or
  # names of R functions
  # param: - if character, pathname of the parameter file
  #        - if list, list of parameters
  #        - if NULL: default values of all parameters
  #        Among these parameters, when param is a list
  #      poly: list of vectors of length 2 or 2-columns matrix:
  #       identifiers of the required pairs of polygons
  #       as they are noted on the polygons file.
  #       Default: all pairs of polygons
  # resfile: pathname of the results file
  #     if NULL, the function returns NULL
  #       else the function returns the main results
  # RETURN
  #  When resfile is NULL: NULL
  #   else the return of the function getRes (see this function)
  # -------------------------------------------------------
# Verif: param doit etre NULL, ou une chaine de car. ou une liste
  if (!is.character(param) && !is.null(param)) {
    if (!is.list(param))
      stop("param should be a filename or a list of parameters")
  }

  # Les fonctions de dispersions
  dispf <- c(dispf) #mettre dans un vecteur
  nfun <- length(dispf)
  if (is.numeric(dispf)) {
    dispfc <- dispf # dispfc: indices des fonctions en C
    dispf <- NULL
  }  else    dispfc <- 0
    
  # Le fichier de stockage des parametres:   nomficp 
  nomficp <- NULL
  ficp <- NULL
  if (is.character(param))
    nomficp <- param # fichier de parametres deja fourni
  else {
    # Ecrire les paramtres sur un fichier
    # temporaire sur le tmp de l'usager
    nomficp <- tempfile(pattern = "califlopp")
    ficp <- file(nomficp, open="w")
    if (!is.null(param))
      mapply( decodparam, param, names(param), ficp)
    # si les fonctions de dispersion ne sont pas des fonctions R
    # mettre les indices de celles voulues
    if (all(dispfc>0)) {
      decodparam(nfun, "nfunc", ficp)
      decodparam(dispfc, "func", ficp)
    }
    close(ficp)
  } # fin else
  
  if (is.null(resfile))
    resfile <- ""
  else
    if (!is.character(resfile))
      stop("resfile should be the pathname of a file")

  # Quand tous les param ont la valeur par dfaut: pas de ficp
    if (is.null(nomficp))
      nomficp<- ""
  # Appeler CaliFloPP:
  .C("CALLcaliflopp",  as.integer(nfun),
     as.character(file),
   as.character(nomficp),
   as.character(resfile),
     as.integer(dispfc),
     dispf,  new.env())
#     dispf,  parent.frame())

  # Dtruire le fichier temporaire des paramtres quand ce n'est pas
  # celui fourni par l'usager:
  if (!is.null(ficp) && !is.character(ficp)) {
      unlink(nomficp)
    }
  

  # Retourner les rsultats s'ils ont t stocks:
  if (resfile != "")
    return(getRes(resfile))
  else
    return(NULL)
}  # fin califlopp
