# ++++++++++++++++++++++++++++++++++++++++++
# Crer et remplir des structures R avec les rsultats
# lus sur un fichier de rsultats de CaliFloPP quand
# le format de celui-ci correspond :
#   OUTPUT_FILE_FORMAT=LIGHT (cf caliclonfig.h)
# AB: 24-09-2009
# ++++++++++++++++++++++++++++++++++++++++++++++
decodcub <- function(X, nfunc) {
  # FUNCTION
  #  Renvoie le contenu d'une ligne de rsultats cubature quand
  #      OUTPUT_FILE_FORMAT=LIGHT dans le fichier caliconfig.h
  # INPUT
  # X: une ligne du fichier  de rsultats concernant une paire
  #    de polygones
  # nfunc: nombre de fonctions de dispersion
  # RETURN
  # Une liste avec les rsultats  cubature  concernant la paire
  #    de polygones, dont les composants sont:
  # mean.flow,  abs.err, n.eval: vecteurs (nfunc)
  # conf.int: matrice (nfunc,2)
  # area, poly: vecteurs (2)
  # --------------------------------------------------------
  iarea <- 3+nfunc
  iconf <- iarea + nfunc+1
  iabs <- iconf+2
  labelfunc <- paste("f", 1:nfunc, sep="")
  poly <- X[1:2]
  ret <- list()
  ret$poly <- paste(poly[1], poly[2], sep="-")
  ret$area <- as.double(X[(iarea):(iarea+1)])
  names(ret$area) <- poly
  iarea <- iarea +2
  to <- iarea+5*(nfunc-1)
  ret$mean.flow <- as.double(X[seq(iarea, to=to, by=5)])
  names(ret$mean.flow) <- labelfunc
  iarea <- iarea +1; to <- iarea+5*(nfunc-1)
  conf.int <- as.double(X[seq(iarea, to=to, by=5)])
  iarea <- iarea +1; to <- iarea+5*(nfunc-1)
  conf.int <- c(conf.int,
                    as.double(X[seq(iarea, to=to, by=5)]))
  conf.int <- matrix(conf.int, ncol=2,
        dimnames= list(labelfunc, c("lower","upper")))
  ret$conf.int <- conf.int
  iarea <- iarea +1; to <- iarea+5*(nfunc-1)
  ret$abs.err <- as.double(X[seq(iarea, to=to, by=5)])
  names(ret$abs.err) <- labelfunc
  iarea <- iarea +1; to <- iarea+5*(nfunc-1)
  ret$n.eval  <- as.integer(X[seq(iarea, to=to, by=5)])
  names(ret$n.eval) <- labelfunc
  return(ret)
} # fin decodcub

# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

decodgrid <- function(ficres, nfunc) {
  # FUNCTION
  #  Renvoie le contenu d'un fichier de rsultats grid quand
  #      OUTPUT_FILE_FORMAT=LIGHT dans le fichier caliconfig.h
  # INPUT
  # ficres: pathname du fichier de rsultats
  # nfunc: nombre de fonctions de dispersion
  # RETURN
  # Une liste avec les rsultats grid dont les composants sont:
  # step,  nr: scalaires
  # autant des composants que de paires de polygones. Chacun est
  # une liste dont les composants sont:
  # poly, area: vecteurs (2)
  # n.eval: scalaires
  # mean.flow, std: vecteurs (nfunc)

  # --------------------------------------------------------
iskip <-1
retour<-list()
# debcouples: true quand on est sur la 1iere ligne
# d'un couple de polys non identiques
debcouple <- TRUE
while(1) {
  ret <- list()
  # 1iere ligne: les idents de polys et les aires
  X <- scan(ficres, skip=iskip, what=character(0), quiet=TRUE)
  if (length(X)==0) break # fin de fichier
  poly <- X[1:2]
  ret$poly <- paste(poly[1], poly[2], sep="-")
  ret$area <- as.double(X[(3+nfunc):(4+nfunc)])
  names(ret$area) <- poly
  iskip <- iskip+1
  # Lorsque les idents de polys ne sont pas les mmes,
  # il n'y a que cette 1iere ligne pour le 1ier couple
  # de la paire
  if (debcouple && (poly[1] != poly[2])) {
    retour[[paste(poly[1], poly[2], sep="-")]] <- ret
    debcouple <- FALSE
    next
  }
  debcouple <- TRUE
  # 2ieme ligne: Les steps et neval
  X <- scan(ficres, skip=iskip, what=character(0), quiet=TRUE)
  ret$step <- as.double(X[1:2])
  ret$n.eval <- as.integer(X[3])
  # Les rsultats par fonction:
  ret$mean.flow <- rep(NA, nfunc)
  names(ret$mean.flow) <- paste("f", 1:nfunc, sep="")
  ret$std <- rep(NA, nfunc)
  names(ret$std) <- paste("f", 1:nfunc, sep="")
  # Lignes suivantes: les rsultats, fonction par fonction
  for (ifunc in 1:nfunc) {
    iskip <- iskip+1
    X <- scan(ficres, skip=iskip, what=character(0), quiet=TRUE)
    ret$mean.flow[ifunc] <- as.double(X[1])
    ret$std[ifunc] <- as.double(X[2])
    ret$nr <- as.integer(X[3])
  } # fin ifunc
  retour[[paste(poly[1], poly[2], sep="-")]] <- ret
  # Lorsque les idents de polys ne sont pas les mmes,
  # il faut complter les rsultats  pour le 1ier couple
  # de la paire
  retour[[paste(poly[2], poly[1], sep="-")]]$n.eval <- ret$n.eval
  retour[[paste(poly[2], poly[1], sep="-")]]$mean.flow <- ret$mean.flow
  retour[[paste(poly[2], poly[1], sep="-")]]$std <- ret$std
  iskip <- iskip+1
} # fin while
# Les steps et nr sont les mmes pour tous les polys: on ne les
# met qu'une fois
if (!is.null(retour[[1]]$step))
    i <-1 else    i<-2
  step <- retour[[i]]$step
  nr <- retour[[i]]$nr
# On les enlve dans chacun des composants
retour <- lapply(retour, function(X) {
  X$step <- NULL; X$nr <- NULL;
  return(X)
})
retour$step <- step
retour$nr <- nr
  return(retour)
} # fin decodgrid

# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
              
getRes <- function(ficres) {
  # FUNCTION
  # Lit un fichier de rsultats de CaliFlopp
  # et renvoie le contenu
  # INPUT
  # ficres: un fichier de rsultats de CaliFlopp
  # OUTPUT
  # A list; as many components as pairs of polygons on the file.
  # Each component is named "poly1-poly2" where "poly1" and "poly2"
  # are respectively the identifiers of the polygons.
  #  Each component is a list which contains:
  # - poly: identifier of the pair, of pattern "poly1-poly2"
  # - area: vector of length 2, area of poly1 and poly2 in
  # square meters
  # - when the method is cubature:
  #   * mean.flow, abs.err, n.eval:
  # scalars or vectors of length equal to the number of
  # dispersion functions
  #   * conf.int: matrix (nfunc,2)
  # - when the method is grid:
  #   * n.eval : scalar
  #   * mean.flow, std:  scalars or vectors of length
  # equal to the number of dispersion functions
  # In addition, the returned list, when the method is grid,
  # contains the components step (vector of length 2) and nr
  # -----------------------------------------------------------
  # Lire  nfunc et methode:
lu <- scan(ficres, nmax=8, what=character(0), quiet=TRUE)
  nfunc <- as.integer(lu[6])
  method <- lu[8]

  switch(method,
         cubature =   {
  # Lire tout le reste:
  lu <- read.table(ficres, header=FALSE, skip=1)
  ret <- apply(lu, 1, decodcub, nfunc)
  for(i in 1:length(ret)) {
    names(ret)[i] <- ret[[i]]$poly
  }
}, # fin cubature
         grid =   {
           ret <- decodgrid(ficres, nfunc)},
         stop(paste("Unknown method", method))
         ) # fin switch
  return(ret)
} # fin getRes

