\name{califlopp}
\alias{califlopp}
\title{CaliFloPP: Calculation of the Integrated Flow of Particles
    between Polygons}
\description{
  Calculation of the flow of particles between polygons by two integration methods: integration by a cubature method and
integration on a grid of points.
}
\usage{
califlopp(file, dispf=c(1,2), param = NULL, resfile = NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{file}{ Pathname of the polygons-file. See details. }
  \item{dispf}{Vector of integers (the dispersion functions are
    then compiled) or vector of R functions (the dispersion functions
    are then R
    functions, provided by the user).
    
    If vector of integers:  indexes of the required
    individual dispersion:
    1 for the dispersion of oilseed rape pollen, 2 for the
    dispersion of oilseed rape seed and 3 for the constant function
    (see the file \code{src/functions.cc}).

    If vector of functions: the dispersion functions are coded in R
    (more time consuming). See details.}
  \item{param}{ Optional list of parameters. Valid components are
    \code{input}, \code{output},
    \code{verbose}, \code{delim},
    \code{poly}, \code{method}, \code{dz}, \code{dp}.
    In addition, when the method is "cub" (cubature): \code{maxpts},
    \code{reler}, \code{abser}, \code{tz}.
    When the method is "grid": \code{seed},
    \code{step}, \code{nr}.
    See details. }
  \item{resfile}{  Optional pathname of a result-file. When set,
    and only when set, the results
    are returned by the function.

    Note: The results  can also be read in R structures
    afterwards by using the function
    \code{\link{getRes}}}
}
\details{
  \bold{The polygons-file}
  
  The coordinates of the polygons should be provided in an ASCII file,
  denoted "polygons-file". The unit is the meter. The vertices should
  be ordered clockwise. The polygons can be closed
  or not, but  without holes. The first line contains the number of polygons.
  The following lines depend of the parameter \code{input}:
  \item{\code{input=1}}{Two lines per polygon: on the first one,
    an identifier (a positive
    integer),   followed by the x-coordinates,
    on the second one, the same identifier
    followed by the y-coordinates.}
  \item{\code{input=2}}{Three lines per polygon: on the first one,
    an identifier (a positive
    integer),   followed by a name for the polygon and
    by the number of its vertices, on the second one, the x-coordinates,
    and on the third one, the y-coordinates.}

  \bold{The individual dispersion functions}
  
  The individual dispersion functions can be either compiled
   or provided by user
  in R functions.
  \item{When compiled,}{the function has one argument,
    the current point, \code{p}, of class \code{Point}.
    Among the methods of this class, we can mention \code{p.getX()}
    and \code{p.getY()} to get the coordinates of the current point
    (in meters*\code{SCALE}, where \code{SCALE} is the  rescaling
    parameter; usually, \code{SCALE}=10 to take into account dm),
    \code{p.dist0()}, the distance of \code{p}
    from the origin (in meters*\code{SCALE})
    and \code{p.angle0()}, the angle (in degrees, in \eqn{[-\pi, +\pi]}{[-PI, +PI]}) between the line  (0,\code{p})
    and the horizontal   line.

  }
  
  \item{When R function,}{the function has one argument,
    the localization of the current point, \code{p}: it is a vector of length
  two, which contains the distance of \code{p} from the origin
  (in meters) and the angle (in degrees, in \eqn{[-\pi, +\pi]}{[-PI, +PI]}) between the line  (0,\code{p})
  and the horizontal   line (if \code{x} and \code{y} are the coordinates
  of \code{p}, the angle is
  \eqn{atan2(y,x)*\frac{180}{\pi}}{atan2(y,x)*180/PI})}

  \bold{The parameters}

  The argument \code{param} is a list which valid components are:
  
  \item{\code{input}}{format of the polygons-file. 1 or 2 (see above).
  Default 2}
  \item{\code{output}}{output required  on the screen:
 0 nothing, 1: all  results, 2:  progression numbers,
 3: the integrated flows and their means per squared meter.
Default 1}
    \item{\code{verbose}}{TRUE, if output is required about polygons convexity
      and landscape translation. Default FALSE}
 \item{\code{delim}}{separator between values in the polygons-file. One
   character. Default: tabulate}
 \item{\code{poly}}{required pairs of polygons.
   List of vectors of length 2, or two-columns  matrix.
   If only one pair is required, it may be a vector of length 2.
 Default: all pairs of polygons.}
   \item{\code{method}}{"cub" for cubature method, "grid" for the grid
     method. Default: "cub"}
   \item{\code{dz}}{integer vector of length equal to the number of
     required dispersion functions. \code{dz[i]} is the distance in meters
     beyond which the dispersion function \code{i} is considered as nul.}
   \item{\code{dp}}{integer vector of length equal to the number of
     required dispersion functions. \code{dp[i]} is the distance in meters
     beyond which the dispersion function \code{i} is calculated between
     centroids only.}
   
   In addition, when the method is "cub" (cubature):
   \item{\code{maxpts}}{ maximal
     number of evaluation points required for each function.
     Vector of length equal to the number of required functions.}
    \item{\code{reler}}{relative error required for each function.
      Vector of length equal to the number of required functions.}
    \item{\code{abser}}{absolute error required for each function.
      Vector of length equal to the number of required functions.}
   \item{\code{tz}}{integer vector of length equal to the number of
     required dispersion functions. Mode of triangulation for the
     cubature method.
Should be 1, if triangulation from (0,0) has to be done
when  (0,0) is included in the integration area and, 0 if not.
1 is recommended  when the dispersion function is very "sharp"
at the origin.}
    
    When the method is "grid":
    \item{\code{seed}}{seed of the random generator.}
    \item{\code{step}}{step of the grid on the x-axis and on the y-axis
      in meter. Vector of length 2.}
    \item{\code{nr}}{maximal number of replications or grids.}
}
\value{
  NULL if \code{resfile} is NULL.
  Otherwise the return of function \code{\link{getRes}}.
}
\section{Side effect}{This function creates a temporary file to store the
  parameters, usually in the directory \code{tmp} of the user. This file
  is destroyed at the end of execution.} 
\references{ http://www.inra.fr/miaj/public/logiciels/califlopp/ }
\author{ A. Bouvier }
\note{ Details can be found in the Reference
  Manual.
  }
  \seealso{  \code{\link{getRes}}}
  \examples{
# Grid method with compiled constant and seed dispersion functions:
param <- list(method="grid",  grid=list(step=c(50,50)))
\dontrun{califlopp("MyPolygonsFile",dispf=c(3,1), param=param)}

# Cubature method with a R dispersion function:
disppollen <- function(X) {
return(fpollen(X[1]))
} # fin disppollen
param <- list( output=1, input=2, dz=0, dp=100, tz=0)
\dontrun{califlopp("MyPolygonsFile", dispf=disppollen, param=param)}
}
\keyword{ optimize }
