/*--------------- COPYRIGHT ------------------------
| INRA - Unit MIA de Jouy en Josas                |
--------------------------------------------------*/

/*--------------- IDENTIFICATION PRODUIT -----------
| Derniere mise a jour : 15 Fev 2006               |
| Role                 : les fonctions de          |
|  dispersion individuelle                         |
--------------------------------------------------*/
///////////////////////////////////////////////////////

#include <math.h>
#include "caliconfig.h"
#include "calitypes.h"
#include "Point.h"

////////////////////////////////////////////////////////////////
// la fonction de dispersion du pollen:
// celle  de Klein modifie par Nathalie Colbach le 11/09/2006
// pour les distances > 50m
////////////////////////////////////////////////////////////////
real
f1 (const Point & p)
{
  real r, rt, rr, gamma, K;
  r = (p.dist0 () / SCALE);

  if (r <= 1.5)
    rt = 0.340 - 0.405 * r + 0.128 * pow (r, 2.0);
  else if (r <= 50)
    rt = 0.03985 / (1.0 + pow (r, 3.12) / 3.80);
  else
    {
      rr = 50;
      gamma = -2.29;
      //valeur la plus lourde -2.14, + lgre - 2.56
      K = (0.03985 / (1.0 + pow (rr, 3.12) / 3.80)) / pow (1.0 + rr, gamma);
      rt = K * pow (1.0 + r, gamma);
    }
  //modif selon Devaux 2006 pour longue distance NC 11 9 2006
  return rt;
}



////////////////////////////
// fonction de dispersion des graines (Colbach et al. 2000b)
////////////////////////////
real
f2 (const Point & p)
{
  real grainesB = 1.38930;
  real grainesC = 2.08686;
  real distance, fd;
  distance = (p.dist0 () / SCALE);
  fd = (grainesB * grainesC) * pow (distance, grainesC - 2) *
    exp (-grainesB * pow (distance, grainesC)) / (2.0 * M_PI);

  return (fd);
}

////////////////////////////
//  Fonction constante qui renvoie tjrs 1
////////////////////////////

real
f3 (const Point & p)
{
  return 1.0;
}

//////////////////////
  // Une gaussienne 
////////////////////////////////////////////////////////////////
real
f4 (const Point & p)
{
  real sigma = 100.0, mu = 0.0;
  real distance, fd, a, b;


  distance = (p.dist0 () / SCALE);

  a = 1.0 / (sigma * sqrt (2 * M_PI));
  b = ((distance - mu) * (distance - mu)) / (2.0 * sigma * sigma);
  fd = a * exp (b);
  return (fd);
}

////////////////////////////
//  Fonctions inutilises 
////////////////////////////

////////////////////////////////////////////////////////////////
// la fonction de dispersion du pollen de Klein
// avant le 13/09/2006
////////////////////////////////////////////////////////////////
real
f6 (const Point & p)
{
  real r, rt;

  r = (p.dist0 () / SCALE);
  if (r <= 1.5)
    rt = 0.340 - 0.405 * r + 0.128 * pow (r, 2.0);
  else
    rt = 0.03985 / (1 + pow (r, 3.12) / 3.80);
  return rt;
}

////////////////////////////////////////////////////////////////
// Une fonction discontinue pour illustrer 
// ce qui se passe dans ce cas.
// Kin Kiu: 7/3/2007
// Cette fonction de dispersion est  support compact et, 
// pour des a,b bien choisis, elle s'annule assez brutalement.
// Du coup, on peut avoir un triangle qui intersecte un peu le 
// support de la fonction de dispersion mais o tous les points 
// de la rgle de cubature sont hors du support.
// L'intgrale sur ce triangle sera considre comme nulle et 
// le triangle ne sera plus divis. 
////////////////////////////////////////////////////////////////

real
f5 (const Point & p)
{
  real distance, a = 10.0, b = 20.0;
  distance = (p.dist0 () / SCALE);
  if (fabs (distance) <= sqrt (a / b))
    {
      return (a - b * distance * distance);
    }
  else
    return 0.0;
}

////////////////////////////////////////////////////////////////
