###############################################################################
# Before use, CUSTOMIZE,
# The compilator:
CC  =   g++ 
# The suffix of the source-files:
SUFFIX = cpp
# Name of the executable name:
EXEC_NAME= califlopp
# The compilation options:
# Careful:  optimization options may lead to execution errors
CFLAGS = -c -fno-implicit-templates

# Useful on systems where the function 'times()' is available:
COPTIONS1 = -DHAVE_TIMES
# Useful on systems where the function 'times()' is not available
# but 'getrusage()':
# COPTIONS1 = -DHAVE_GETRUSAGE 


# To switch the random numbers generator to 'drand48' (Unix):
#  COPTIONS2 =  -DHAVE_DRAND48
# To switch the random numbers generator to 'rand':
#  COPTIONS2 =  -DHAVE_RAND
# To switch the random numbers generator to 'genrand_real2'
COPTIONS2 =  -DHAVE_GENRAND_REAL2

# End of CUSTOMIZE
# -------------------------------------------


COPTIONS = $(COPTIONS1) $(COPTIONS2)
# Note: ROOT is defined in the makefile
SRCDIR = $(ROOT)/src
ADAPTDIR=$(SRCDIR)/adapt
INCL=-I$(ROOT) -I$(SRCDIR) -I$(ADAPTDIR) 

# Si on  veut  les ip de debogage
#DEBUG = -DDEBUG
# To test: make sure everything compiles cleanly:
# output all warnings
#CFLAGS =-c -fno-implicit-templates -Wno-deprecated -ansi -W -Wall -Wconversion  -pedantic -Wpointer-arith  -Wcast-align -Wfloat-equal  -Wextra 

# Add inputs and outputs from these tool invocations to the build variables 


C_ADAPT_SRCS = \
$(ADAPTDIR)/Adapt.$(SUFFIX) \
$(ADAPTDIR)/PileTr.$(SUFFIX) \
$(ADAPTDIR)/Point.$(SUFFIX) \
$(ADAPTDIR)/Rule.$(SUFFIX) \
$(ADAPTDIR)/Triangle.$(SUFFIX)  \
$(ADAPTDIR)/Vector.$(SUFFIX)


C_OTHER_SRCS = \
$(SRCDIR)/crConvexSp.$(SUFFIX) \
$(SRCDIR)/methodAdapt.$(SUFFIX) \
$(SRCDIR)/methodGrid.$(SUFFIX) \
$(SRCDIR)/util.$(SUFFIX) \
$(SRCDIR)/fluxad.$(SUFFIX) \
$(SRCDIR)/fluxsd.$(SUFFIX) \
$(SRCDIR)/functions.$(SUFFIX) \
$(SRCDIR)/geom.$(SUFFIX) \
$(SRCDIR)/go.$(SUFFIX) \
$(SRCDIR)/intersection.$(SUFFIX) \
$(SRCDIR)/main.$(SUFFIX) \
$(SRCDIR)/mt19937ar.$(SUFFIX) \
$(SRCDIR)/readPoly.$(SUFFIX) \
$(SRCDIR)/read1Poly.$(SUFFIX) \
$(SRCDIR)/timing.$(SUFFIX) \
$(SRCDIR)/writeDebug.$(SUFFIX) \
$(SRCDIR)/zoneintegration.$(SUFFIX) 

C_UPPER_SRCS = $(C_ADAPT_SRCS) $(C_OTHER_SRCS)


ADAPT_OBJS = \
./Adapt.o   ./Point.o  \
./PileTr.o  ./Rule.o   ./Triangle.o ./Vector.o


OTHER_OBJS = \
./crConvexSp.o \
./methodAdapt.o \
./methodGrid.o \
./util.o \
./fluxad.o \
./fluxsd.o \
./functions.o \
./geom.o \
./go.o \
./intersection.o \
./main.o \
./mt19937ar.o \
./readPoly.o \
./read1Poly.o \
./timing.o \
./writeDebug.o \
./zoneintegration.o 

OBJS = \
$(ADAPT_OBJS)  $(OTHER_OBJS)

OBJSURF = \
./surface.o  ./util.o ./readPoly.o ./read1Poly.o ./geom.o ./crConvexSp.o

OBJAIREI  = \
./aireintersect.o  ./util.o ./readPoly.o ./read1Poly.o ./geom.o \
./crConvexSp.o \
./intersection.o ./writeDebug.o

OBJALIGN = \
./polysimplif.o ./util.o  ./readPoly.o ./read1Poly.o ./geom.o \
./crConvexSp.o


DEPS = \
${addprefix ./, \
Adapt.d   Point.d  PileTr.d  Rule.d   Triangle.d Vector.d \
timing.d \
crConvexSp.d \
methodAdapt.d \
methodGrid.d \
aireintersect.d \
util.d \
fluxad.d \
fluxsd.d \
functions.d \
geom.d \
go.d \
intersection.d \
main.d \
readPoly.d \
polysimplif.d \
surface.d \
writeDebug.d \
zoneintegration.d  \
}


# Each subdirectory must supply rules for building sources it contributes
%.o: $(ADAPTDIR)/%.$(SUFFIX)          
	@echo 'Building adapt file: $<'
	@echo 'Invoking: GCC C++ Compiler'
	@echo $(CC)   $(CFLAGS) $(INCL) $(COPTIONS) -o$@ $<
	@$(CC)  $(CFLAGS) $(INCL) $(COPTIONS)  -o$@ $< && \
	echo -n $(@:%.o=%.d) $(dir $@) > $(@:%.o=%.d) && \
	$(CC) -MM -MG -P  $(CFLAGS)  $(INCL) $(COPTIONS)  $< >> $(@:%.o=%.d)
	@echo 'Finished building: $<'
	@echo ' '



%.o: $(SRCDIR)/%.$(SUFFIX)          
	@echo 'Building file: $<'
	@echo 'Invoking: GCC C++ Compiler'
	@echo $(CC)  $(CFLAGS) $(INCL) $(COPTIONS) -o$@ $<
	@$(CC)  $(CFLAGS)  $(INCL) $(COPTIONS)  -o$@ $< && \
	echo -n $(@:%.o=%.d) $(dir $@) > $(@:%.o=%.d) && \
	$(CC) -MM -MG -P  $(CFLAGS) $(INCL) $(COPTIONS)  $< >> $(@:%.o=%.d)
	@echo 'Finished building: $<'
	@echo ' '


