/*--------------- COPYRIGHT ------------------------
| INRA - Unit MIA de Jouy en Josas                |
--------------------------------------------------*/

/*--------------- IDENTIFICATION PRODUIT -----------
| Derniere mise a jour : 15 Fev 2006               |
| Role                 : gre le dialogue avec     |
|                        l'utilisateur             |
|  (version interactive)                           |
|    Interactive version: dialog with the user     |
|    No argument;                                  |
|    Return: OK or a negative error code.          |
--------------------------------------------------*/
///////////////////////////////////////////////////////
int califlopp_ad ();


///////////////////////////////////////////////////////
// Version avec dialogue
///////////////////////////////////////////////////////

#include "caliinclc.h"
#include "calitypes.h"
#include "caliconfig.h"
#include "caliboolean.h"
#include "calierror.h"
#include "calimacros.h"
#include "go.h"
#include "methodAdapt.h"
#include "methodGrid.h"
#include "methodIntegr.h"
#include "readPoly.h"
#include "util.h"

//////////////////////////////////////////////////////////
// Liberer la memoire
//////////////////////////////////////////////////////////
void
libMem (int npoly,
	int *choisi,
	int *a, int *numPoly, real * area,
	int **ni, char **nomPoly, tPolygoni ** Poly, real ** bary)
{

  libMemPoly (npoly, a, numPoly, area, ni, nomPoly, Poly, bary);

  DETRU_T1 (choisi);

}				// fin libMem

//////////////////////////////////////////////////////////
// Saisie de la methode et ses arguments
/////////////////////////////////////////////////////////
int
LireMethode (Boolean & grid)
{
  int adapt;
  char moi[] = "LireMethode";
/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char errlect[]="Error in reading";


  // choisir la mthode
  printf ("Choose the method: 0 for grid, 1 for adaptative cubature\n");
  if (scanf ("%d", &adapt) <1) {
    return (ecrmess (CALI_ERLECT, moi, errlect, True));
  }

  if ((adapt < 0) || (adapt > 2))
    {
      sprintf (errmess, "invalid function number; must be in [0-1] \n");
      return (ecrmess (CALI_ERDIAG5, moi, errmess));
    }

  if (adapt == 1)
    {
      grid = False;
    }
  else
    {
      grid = True;
    }
  return (0);
}				// fin  LireMethode



//////////////////////////////////////////////////////////
// Enchainement de la saisie de la fonction, de la methode et ses arguments
/////////////////////////////////////////////////////////
methodIntegr *
Choisir (methodGrid & agrid, methodAdapt & aadapt, int nfunc, int *ifunct,
	 Boolean & grid, int &code)
{

  methodIntegr *lamethode;
  // Choisir la methode:
  if ((code = LireMethode (grid)) != 0)
    return (NULL);

  // Construire l' interface  la methode
  if (grid)
    {
      agrid = methodGrid (nfunc, ifunct);
      lamethode = &(agrid);

      //      printf ("\nCalculation by the 'grid' method\n\n");
    }
  else
    {
      aadapt = methodAdapt (nfunc, ifunct);
      lamethode = &(aadapt);
      // printf ("\nCalculation by using the 'adapt' library\n\n");
    }

  if ((code = lamethode->ReadArgu ()) != 0)
    return (NULL);
  printf ("\n");

  return (lamethode);
}				// fin Choisir

///////////////////////////////////////////////////////
// Le programme maitre
///////////////////////////////////////////////////////
int
califlopp_ad ()
{

  // Paramtres des mthodes
  int cas;
  Boolean grid;			// l'indicateur de la mthode voulue

  // Les mthodes
  methodIntegr *methode;
  methodAdapt aadapt;
  methodGrid agrid;


  // Pour le dialogue avec l'utilisateur
  char reponse[2];
  int clu, dlu, nchoisi;
  int *choisi = NULL;
  int *target;

//////////////////////////////////////////////////////////
  // Declaration des paramtres:
//////////////////////////////////////////////////////////

  // Paramtres des mthodes
  // True si on veut l'ip des coordonnes lues
  // et de l'indication si les poly sont convexes ou non  
  Boolean pverbose = Boolean (DEFAULT_VERBOSE);

  // Grid:
  real pstepx = real (DEFAULT_STEPX);
  real pstepy = real (DEFAULT_STEPY);
  int pnr = int (DEFAULT_EST);
  unsigned int pseed = DEFAULT_SEED;

  // Adapt:
  real preler[MAX_NFUNCTIONS], pabser[MAX_NFUNCTIONS];
  long int preqmaxpts[MAX_NFUNCTIONS];

//////////////////////////////////////////////////////////


  // Les polygones:
  int npoly;



// Va de travail
  int i, erreur;
  char lu[MAX_LINE_POLY];
  // Pour lire les polygones puis pour stocker des rsultats:
  // PATH_MAX est defini dans stdio.h
  char filenamei[PATH_MAX];	// pathname of the input-file
  char filenamer[PATH_MAX];	// pathname of the result-file
  char filenameadapt[PATH_MAX], filenamegrid[PATH_MAX];
  FILE *fpi;
  char *pfilenamer;

  char moi[] = "califlopp_ad";

/* Pour stocker les messages */
  char errmess[CHAR_MAX];
  char errlect[]="Error in reading";



//////////////////////////////////////////////////////////

  // On ne  demande pas les paramtres suivants
  // pour ne pas alourdir le dialogue.
  // Le paramtre grant les sorties sur l'cran:
  int poutput = DEFAULT_OUTPUT;
  int pinput = DEFAULT_INPUT_FORMAT;
  char pdelim[] = DEFAULT_DELIM;
  int nfunc = DEFAULT_NFUNCTIONS;
  int ifunct[MAX_NFUNCTIONS];
  long int reqmaxpts[MAX_NFUNCTIONS];

  for (i = 0; i < nfunc; i++)
    {
      ifunct[i] = (i + 1);
      reqmaxpts[i] = 0;
    }

//////////////////////////////////////////////////////////
// Les options de lecture et ip
//////////////////////////////////////////////////////////

  printf ("Type the pathname of the polygons file:");
  if (scanf ("%s", filenamei)<1) {
        return (ecrmess (CALI_ERLECT, moi, errlect, True));
  }

  printf ("Do you want verbose reading? (y/n)");
  if (scanf ("%s", reponse)<1) {
       return (ecrmess (CALI_ERLECT, moi, errlect, True));
  }
  if (*reponse == 'y')
    pverbose = True;
  else
    pverbose = False;



//////////////////////////////////////////////////////////
  // Lecture du fichier de polygones
//////////////////////////////////////////////////////////

  fpi = fopen (filenamei, "r");
  if (!fpi)
    {
      sprintf (errmess, "cannot open polygons file %s\n", filenamei);
      return (ecrmess (CALI_ERFIC1, moi, errmess));
    }

  // Lire avec fgets ce fichier, car il a un nbre va. de donnes
  // sur chaque ligne et il ne faut pas mlanger les fscanf et fgets
  if (fgets (lu, MAX_LINE_POLY, fpi)==NULL) {
     return (ecrmess (CALI_ERLECT, moi, errlect, True));
  }
  npoly = atoi (lu);
  if ((poutput != NOTHING) && (poutput != LIGHT))
    {
      printf ("CaliFloPP -  Copyright (c) 2007 - INRA\n");
      printf ("\nNumber of polygons: %d\n-------------------\n", npoly);
    }



//////////////////////////////////////////////////////////
  // Declaration des structures de dim npoly:
//////////////////////////////////////////////////////////
  int *a;
  real *area;
  int **ni;
  tPolygoni **Poly;
  int *numPoly;			/* les nos d'identification des polys */
  char **nomPoly;		/* les noms des polys */
  real **bary;			/* Les barycentres des polys */

//////////////////////////////////////////////////////////
  // Allocation des structures de dim npoly:
// memory allocation of the structures of dimension=npoly
///////////////////////////////////////////////////////
  CREER_T1 (a, npoly, int);
  CREER_T1 (numPoly, npoly, int);
  CREER_T1 (area, npoly, real);
  CREER_T2 (ni, npoly, int);
  CREER_T2 (nomPoly, npoly, char);
  CREER_T2 (Poly, npoly, tPolygoni);
  CREER_T2 (bary, npoly, real);

  for (i = 0; i < npoly; i++)
    {
      CREER_T1 (ni[i], MAX_TRIANGLES, int);
      CREER_T1 (nomPoly[i], MAX_NAME, char);
      CREER_T1 (Poly[i], MAX_TRIANGLES, tPolygoni);
      CREER_T1 (bary[i], DIM, real);
    }



//////////////////////////////////////////////////////////
// Lecture des polygones
//////////////////////////////////////////////////////////
  Boolean calcSurf = False;	// on ne veut pas que les surfaces
  // (calcSurf n'est vrai que pour le programme surface)

  tPolygond **Polyd = NULL;	// on ne veut pas les polys corrigs
  // (Polyd n'est pas NULL que pour le programme polysimplif)


  int npolybons;
  erreur = ReadPoly (fpi, pverbose, calcSurf, pinput, pdelim,
		     npoly, nomPoly, numPoly, npolybons,
		     a, area, ni, Poly, Polyd, bary);

  if ((erreur != OK) && (erreur != CALI_WARNPOLY))
    {
      libMem (npoly, choisi, a, numPoly, area, ni, nomPoly, Poly, bary);
      return (erreur);		// fatal error
    }

  if (npolybons == 0)
    {
      libMem (npoly, choisi, a, numPoly, area, ni, nomPoly, Poly, bary);

      sprintf (errmess, "All polygons are invalid\n");
      return (ecrmess (CALI_ERPOLY7, moi, errmess));
    }



//////////////////////////////////////////////////////////
// Boucle de dialogue
//////////////////////////////////////////////////////////
  char boucler[2];
  char openr[2];
  int code, fois = 1;
  strcpy (boucler, "y");
  strcpy (openr, " ");

  while (*boucler == 'y')
    {

/////////////////////////////////////////////////
// Choisir une mthode et ses paramtres
/////////////////////////////////////////////////
      methode = Choisir (agrid, aadapt, nfunc, ifunct, grid, code);

      if (code != 0)
	{
	  libMem (npoly, choisi, a, numPoly, area, ni, nomPoly, Poly, bary);
	  return (CALI_ERCRMETH);
	}

      if (fois == 1)
	{
	  // ne poser cette question que la 1iere fois:
	  printf ("Do you want to create a result-file? (y/n)");
	  // Initiliser les noms des fichiers
	  strcpy (filenamegrid, " ");
	  strcpy (filenameadapt, " ");
	  if (scanf ("%s", reponse)<1) {
	    return (ecrmess (CALI_ERLECT, moi, errlect, True));
	      }
	  if (*reponse == 'y')
	    {
	      printf (" - pathname of the output file(s):");
	      if (scanf ("%s", filenamer)<1) {
		return (ecrmess (CALI_ERLECT, moi, errlect, True));
	      }
	      strcpy (openr, "w");	// ici, indique qu'on veut un fichier
	    }			// fin (reponse == 'y')
	  else
	    {
	      pfilenamer = NULL;
	      strcpy (openr, " ");
	    }
	}			// fin fois
      fois++;

      // Le fichier de sortie
      if (strncmp (openr, " ", 1) != 0)
	{
	  if (grid)
	    {
	      if (strncmp (filenamegrid, " ", 1) == 0)
		{
		  // fichier pas encore cr
		  strcpy (filenamegrid, filenamer);
		  strcat (filenamegrid, ".grid");
		  strcpy (openr, "w");	// mode criture
		}
	      else
		{
		  // fichier dj cr
		  strcpy (openr, "a");	// mode append
		}
	      pfilenamer = filenamegrid;
	    }			// fin grid
	  else
	    {
	      if (strncmp (filenameadapt, " ", 1) == 0)
		{
		  // fichier pas encore cr
		  strcpy (filenameadapt, filenamer);
		  strcat (filenameadapt, ".adapt");
		  strcpy (openr, "w");	// mode criture
		}
	      else
		{
		  // fichier dj cr
		  strcpy (openr, "a");	// mode append
		}
	      pfilenamer = filenameadapt;
	    }			// fin else adapt

	}			// fin openr





      printf ("Choose which integrated flows are required:\n");
      printf ("1. between two polygons;\n");
      printf ("2. between all pairs of polygons;\n");
      printf ("3. between chosen polygons and all the others.\n");
      if (scanf ("%d", &cas)<1) {
	 return (ecrmess (CALI_ERLECT, moi, errlect, True));
  }


      switch (cas)
	{
	case 1:
	  {
	    /* Traitement d'un seul couple de polys  la fois */
	    printf ("Choose:\n");
	    printf
	      (" ident (numerical ident)  of the first polygon of the pair: ");
	    if (scanf ("%d", &clu)<1) {
	      return (ecrmess (CALI_ERLECT, moi, errlect, True));
	    }
	    printf (" ident of the second polygon: ");
	    if (scanf ("%d", &dlu)<1) {
	      return (ecrmess (CALI_ERLECT, moi, errlect, True));
	    }
	    break;
	  }
	case 2:
	  break;		//c'est par dfaut
	case 3:
	  {
// calcul sur des paires de parcelles choisies

	    printf ("Results from chosen polygons to/from all the others:\n");
	    printf ("Number of chosen polygons?\n");
	    if (scanf ("%d", &nchoisi)<1) {
	          return (ecrmess (CALI_ERLECT, moi, errlect, True));
	    }
	    printf
	      (" type the %d numerical identifiers of the polygons separated by blank characters: ",
	       nchoisi);
	    CREER_T1 (choisi, nchoisi, int);
	    for (i = 0; i < nchoisi; i++)
	      {
		if (scanf ("%d", &clu)<1) {
		   return (ecrmess (CALI_ERLECT, moi, errlect, True));
		}
		choisi[i] = clu;
	      }

	  }			// fin cas3

	}			// fin switch


      ///////////////////////////////////////////
      // Affecter les parametres:
      ///////////////////////////////////////////
      if (grid)
	{
	  // Grid:
	  agrid.getArgu (pstepy, pstepx, pnr, pseed);
	}
      else
	{
	  // Adapt:
	  aadapt.getArgu (preler, pabser, preqmaxpts);

	}

      ///////////////////////////////////////////
      // Lancer l'excution
      ///////////////////////////////////////////
      erreur =
	suite (cas, pverbose, pinput, poutput, grid,
	       pstepx, pstepy, pnr, pseed,
	       preler, pabser,
	       preqmaxpts,
	       nfunc, ifunct,
	       npoly, clu, dlu,
	       nchoisi, choisi, target,
	       a, area, bary,
	       ni, Poly,
	       numPoly, nomPoly, filenamei, pfilenamer, openr, methode);

      if (erreur != OK)
	printf ("\nReturn code: %d\n", erreur);

      ///////////////////////////////////////////
      // Boucler
      ///////////////////////////////////////////


      printf ("\n\nAnother execution? (y/n)");

      if (scanf ("%s", boucler)<1) {
	    return (ecrmess (CALI_ERLECT, moi, errlect, True));
      }

    }				// fin boucler


  ///////////////////////////////////////
  // Librer la mmoire
  ///////////////////////////////////////
  libMem (npoly, choisi, a, numPoly, area, ni, nomPoly, Poly, bary);

  ///////////////////////////////////
  // Indiquer les fichiers crs
  ///////////////////////////////////

  if (strncmp (openr, " ", 1) != 0)
    {
      printf ("\n\nFile(s) created:\n");
      if (strncmp (filenamegrid, " ", 1) != 0)
	printf (" %s\n", filenamegrid);
      if (strncmp (filenameadapt, " ", 1) != 0)
	printf (" %s\n", filenameadapt);
    }

///////////////////////////////////
  return (OK);
///////////////////////////////////

}				// fin califlopp_ad
