##################################################
# Fonction de dcoupage d'un objet de la classe Polygons
# en sous-polygones convexes
#################################################

# #########################################################

if (!isGeneric("crConvex"))
	setGeneric("crConvex",
 function(x, verbose=FALSE, digits=2,
		  maxtriangles=60,
		  maxvertices=50,
		   plot=FALSE,ask=TRUE, ...)
		standardGeneric("crConvex"))
# ____________________________________
# Cration de la dfinition par dfaut 'crConvex'
# ____________________________________
crConvex.default = function(x, verbose=FALSE, digits=2,
		  maxtriangles=60,
		  maxvertices=50,
		   plot=FALSE,ask=TRUE, ...)
  {
  if (!inherits(x,"Polygons")  && !inherits(x,"Polygon"))
    stop("This function is valid only on a 'Polygons' or 'Polygon' object.")
if (inherits(x,"Polygons"))
   crConvex.Polygons(
x, verbose, digits, maxtriangles, maxvertices,
plot, ask,...)
else
crConvex.Polygon(
x, verbose, digits, maxtriangles, maxvertices,
plot, ask,...)	
  }
# -----------------------------------------------
#-------------------------------------------------------
# crConvexP: Fonction de creation des sous-polys convexes
# x: un poly
# id: identificateur  du poly  pour les messages
# Retour: un Polygons  qui contient la liste des Polygon convexes
# et les diagonales essentielles de x
# -------------------------------------------------------

crConvexP = function(x, verbose,  id, digits, maxtriangles, maxvertices)
{
# mettre les coordonnees clock si elles ne le sont pas
# i.e si c'est  un trou
p1 = putClock(x)


# Appeler le C 
# -------------
verbose=as.integer(verbose)
a = slot(p1, "coords")
# ter la fermeture du poly et multiplier par 10**digits
p2=matrix(NA, nrow=(nrow(a)-1), ncol=2)
for (i in 1:(nrow(a)-1))
 p2[i,]=a[i,]*(10**digits)

 nia = as.integer(nrow(p2))
convex=0
nconvex=0
ndiago=0

# Preparer les sorties:
ni = rep(0,maxtriangles) # les nbre de sommets 
polyx=rep(NA,maxtriangles*maxvertices)
polyy=rep(NA,maxtriangles*maxvertices)
diagonumfrom = rep(NA,maxtriangles)
diagonumto = rep(NA,maxtriangles)
diagofrom=rep(NA,maxtriangles*2)
diagoto=rep(NA,maxtriangles*2)

ret = .C("crconvex", as.integer(maxtriangles),
    as.integer(maxvertices),
    as.character(id), as.integer(verbose),
    as.double(p2),     as.integer(nia),
    convex = as.integer(convex),
    nconvex=as.integer(nconvex),
    ni=as.integer(ni),
    ndiago= as.integer(ndiago),
    polyx=as.integer(polyx),
    polyy=as.integer(polyy),
    diagonumfrom=as.integer(diagonumfrom),
    diagonumto=as.integer(diagonumto),
    diagofrom=as.double(diagofrom),
    diagoto=as.double(diagoto),
NAOK=TRUE,  PACKAGE="landm")




# Retourner  le poly en entree s'il est convexe
#-----------------------------------------
if (ret$convex==1) {
return(ConvexPolygon(x, paste(as.character(id), "1", sep="-")))
}


# Sinon, retourner la liste des polygones convexes
#------------------------------------------------

if (ret$nconvex >0)
{
polyconvex=list()
for (ipoly in 1:ret$nconvex)
{
lesx=ret$polyx[seq(ipoly, length.out=ret$ni[ipoly],by=maxtriangles)]
lesy=ret$polyy[seq(ipoly, length.out=ret$ni[ipoly],by=maxtriangles)]
lesx=lesx[!is.na(lesx)]
lesy=lesy[!is.na(lesy)]
# Fermer le poly
lesx=c(lesx, lesx[1])
lesy=c(lesy, lesy[1])
# Les coordonnees sont anticlock en sortie;
# les inverser si le poly en entree n'est pas un trou

        if (slot(x, "hole") == FALSE) {
lesx=rev(lesx)
lesy=rev(lesy)
}
# Diviser par l'echelle:
lesx=lesx/(10**digits)
lesy=lesy/(10**digits)

polyconvex[[ipoly]] = ConvexPolygon(cbind(lesx, lesy),
ID=paste(as.character(id),as.character(ipoly), sep="-"))
} # fin ipoly
} # fin ret$nconvex
else 
{
#  si erreur
polyconvex=NULL
warning(
paste("Error when creating the convex subpolygons on the polygon",
id))
return(NULL)
}

# multiplier par 10**digits les coordonnes des diagonales
#  essentielles
diagofrom=matrix(ret$diagofrom/(10**digits), ncol=2)
diagoto=matrix(ret$diagoto/(10**digits), ncol=2)
return(list(polyconvex=polyconvex, nconvex=ret$nconvex,
				   ndiago=ret$ndiago,
				   diagonumfrom=ret$diagonumfrom,
				   diagonumto=ret$diagonumto,
				   diagofrom=diagofrom,
				   diagoto=diagoto))

} # fin crConvexP


# ____________________________________
# Cration de la dfinition spcialise de 'crConvex'  Polygons
# ____________________________________
# Arguments:
# verbose:T si verbose
# plot: T si, pour chaque poly nonconvexe, on veut son trac et
# celui de ses sous-polys
# ask: si plot, demander une pause avant chaque graphique
# Retour: 
# - un object Polygons, d'identificateur gal  celui du poly en entre
# auquel on ajoute "-convex", qui contient la liste des polys
# convexes.
# Chaque composant de cette liste est un objet "ConvexPolygon",
# qui contient un slot ID, egal 
#<indice-du-poly-pere>-<indice-du-sous-poly-de-celui-ci>
# ---------------------------------------------------------------

crConvex.Polygons=
 function(x, verbose=FALSE, digits=2,
		  maxtriangles=60,
		  maxvertices=50,
		   plot=FALSE,ask=TRUE, ...)

{
if (plot && ask) {
oldpar=par()$ask

par(ask=ask)
}


retour=list()
inum=1

for (unpoly in slot(x, "Polygons")) {

a=crConvexP(unpoly, verbose,
    as.character(inum), digits,
     maxtriangles,
     maxvertices)


if ( !is.null(a))
{

if (inherits(a, "ConvexPolygon"))
{
# le poly est deja convexe
nbconvex =0
retour[[inum]]=list(a)
}
else
	{
nbconvex =a$nconvex
	if (a$nconvex >0)
    retour[[inum]]= a$polyconvex
}


if   (plot==TRUE)
{

# Trace des sous-polys
if (nbconvex>0)
{
plot(Polygons(retour[[inum]],  as.character(inum)), ...)

# Trac du poly origine
plot(unpoly,  add=TRUE, col="red")
}
else
	plot(unpoly,   col="red")


# Rajout des diagonales
if (nbconvex>0) 
{
for (idiag in 1:a$ndiago)
{
	lines(x=c(a$diagofrom[idiag,1], a$diagoto[idiag,1]),
	y=c(a$diagofrom[idiag,2], a$diagoto[idiag,2]),
	 col="blue")
# Labeller les sommets des diago:
text(x=a$diagofrom[idiag,1], y= a$diagofrom[idiag,2],
    labels=as.character(a$diagonumfrom[idiag]), pos=4,
 col="blue",cex=0.7, offset=0.3)

text(x=a$diagoto[idiag,1], y= a$diagoto[idiag,2],
    labels=as.character(a$diagonumto[idiag]), pos=4,
col="blue",cex=0.7 , offset=0.3)
}
} # fin (nbconvex>0) 
} # fin (plot==TRUE)
} # fin a pas null

inum=inum+1
} # fin unpoly


if (plot && ask) 
   par(ask=oldpar)


# Creation du retour:
if (length(retour)>0)
return(Polygons(unlist(retour), paste(slot(x,"ID"), "convex",
sep="-")))
else
return(NULL)

} # fin crConvex.Polygons

setMethod("crConvex", "Polygons", crConvex.Polygons)

# ____________________________________
# Cration de la dfinition spcialise de 'crConvex'  Polygon
# Retour: un Polygons  qui contient la liste des Polygon convexes
# -------------------------------------------------------
crConvex.Polygon=
 function(x, verbose=FALSE, digits=2,
		  maxtriangles=60,
		  maxvertices=50,
		   plot=FALSE,ask=TRUE, ...)

{
b=Polygons(list(x), "1")
return(crConvex.Polygons(b, verbose=verbose, digits=digits, 
			    maxtriangles=maxtriangles,
			    maxvertices=maxvertices,
			    plot=plot, ask=ask, ...))
}

setMethod("crConvex", "Polygon", crConvex.Polygon)

