\name{crConvex.Polygons}
\alias{crConvex.Polygons}
\title{ Split list of polygons into convex subpolygons }
\description{
Split  all the non convex polygons
of an object of \code{\link[sp]{Polygons-class} of package sp}
into convex ones. }
}
\usage{
crConvex.Polygons(x, verbose = FALSE, digits = 2, maxtriangles = 60, maxvertices = 50, plot = FALSE, ask = TRUE, ...)
}
\arguments{
  \item{x}{ An object of class \code{\link[sp]{Polygons-class}} of package sp }
  \item{verbose}{ Logical; when TRUE, details on the 
splitting process are given  }
  \item{digits}{ The number of decimal digits to take into account
for the coordinates }
  \item{maxtriangles}{ The maximal number of convex subpolygons per
  polygon } 
  \item{maxvertices}{ The maximal number of vertices per convex
  subpolygon }
  \item{plot}{  Logical; when TRUE, each non convex polygon and its
convex subpolygons are plotted }
  \item{ask}{  Logical; when TRUE  and when plot is TRUE, the user is
  asked for prompt before each plot }
  \item{\dots}{  when plot is TRUE, arguments passed to \code{\link[graphics]{plot.default}} }
}
\value{
An object of class \code{\link[sp]{Polygons-class}} of package sp where
all the polygons are convex.

The slot ID of this object is equal to
the slot ID of \code{x}, the string "-convex" added.

The slot Polygons contains
objects of \code{\link[sp]{Polygon-class}}
where a slot ID has been added, on  the pattern
"a-b", where a is the index of the polygon from which
the current polygon has been created (its "father")
and b is its index
among the list of all its brothers.
}
\author{  A. Bouvier }
\seealso{ \code{\link{crConvex.Polygon}} }
\examples{
p1=Polygon(cbind(c(540272, 540274, 540278, 540156, 540151, 
       540247, 540272),
  c(1794890, 1794850, 1794800, 1794820, 1794840, 1794870, 1794890)))
p2=Polygon(cbind(c( 540261, 540378, 540374, 540362, 540318, 540315, 
540301, 540304, 540311, 540361,
540357, 540336, 540323, 540274, 540272, 540273, 540261),
c(1795000, 1795000, 1794890, 1794880, 1794890, 
1794970, 1794970, 1794910, 1794880,
1794880, 1794790, 1794790, 1794860, 1794850, 1794890, 
1794920, 1795000)))

lp=Polygons(list(p1,p2), "L")
d=crConvex(lp)
plot(d)

# Another example
p1=Polygon(cbind(c(290766.3, 290776.3, 290776.3, 290786.3, 290674.2, 
290566.3, 290551.9, 290535.0, 290507.7, 290486.3, 290486.3, 290491.2, 
290560.7, 290626.3, 290636.3, 290656.3, 290660.7, 290671.8, 290690.8, 
290739.6, 290756.3, 290766.3, 290766.3),
c(137627.5, 137627.5, 137617.5, 137597.5, 137584.3, 137577.5, 137613.2,
 137626.2, 137661.6, 137667.5, 137677.5, 137727.5, 137783.1, 137787.5, 
137767.6, 137757.5, 137752.0, 137743.1, 137711.8, 137650.9, 137637.5, 
137637.5, 137627.5)))
lp=Polygons(list(p1), "L")
d=crConvex(lp, plot=TRUE, verbose=TRUE)
}
\keyword{manip}
\keyword{methods}
