/*--------------- COPYRIGHT ------------------------
| INRA - Unité MIA de Jouy en Josas                |
--------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include <stdlib.h>

////////////////////////////////////////////////////
// Les includes
////////////////////////////////////////////////////
#include "lesdefines.h"
#include "lestypes.h"
#include "lesmacros.h"
#include "geom.h"



/////////////////////////////////////////////////////////////////
// Note: en R les lignes d'une meme colonne sont stockees 
// consecutivement en memoire.
#define Adp(a, b) ( Ad[nla * (b) + (a)])
#define Bdp(a, b) ( Bd[nlb *  (b) + (a)])



/////////////////////////////////////////////////////////////////
// Fonction de comparaison de reels  Ã  Tolerance pres
/////////////////////////////////////////////////////////////////
int
doubleequal (double a, double b)
{
  if (fabs (a - b) < (double)  DBL_MIN)
    return 1;
  else
    return 0;
}

/////////////////////////////////////////////////////////////////
//          FONCTION CALCULANT LA PLUS PETITE DISTANCE ENTRE
//             DEUX PARCELLES
//          CALCULATING THE MINIMAL DISTANCE BETWEEN TWO PLOTS
// Polygons should be distincts and convex.
// Polygons should be closed:
//  the last vertice should be equal to the first one.
// Vertices should be ordered counterclockwise.
// Ad, Bd: the polygons; matrix nvertices,DIM 
// nia, nib: number of vertices -1
/////////////////////////////////////////////////////////////////

void
DistanceMinimale (double *Ad, int *nia,
		  double *Bd, int *nib,
		  double *distanceMinimale)
{
  int i, j, nl;
  double distance;
  double xp, yp;
  double a[DIM], b[DIM];
 int nla = *nia;
 int nlb = *nib;
 *distanceMinimale=0.0;



  //pour chacun des  c´t©s d'une parcelle, on va calculer la distance
  //minimale avec les  c´t©s de l'autre parcelle
  //for each of the  lines of a plot, the minimal distances with each line
  //of the second plot is calculated
  for (i = 0; i < (*nia -1); i++)
    for (j = 0; j < (*nib-1); j++)
      {

	//recherche de la distance de projection minimale
	//looking for the minimum projection distance

	//droite PjPj+1 non verticales
	//PjPj+1 line not vertical

	//    if (Bdp(j + 1, XX) != Bdp(j, XX))
	if (doubleequal (Bdp(j + 1, XX), Bdp(j, XX)) == 0)
	  {
	    a[1] = (Bdp(j + 1, YY) - Bdp(j, YY)) / (Bdp(j + 1, XX) - Bdp(j, XX));
	    b[1] =
	      (Bdp(j, YY) * Bdp(j + 1, XX) -
	       Bdp(j + 1, YY) * Bdp(j, XX)) / (Bdp(j + 1, XX) - Bdp(j, XX));

	    //abscisse de la projection de Pi sur la droite a1*x+b1
	    //x value of the projection of point Pi to the line a1*x+b1
	    xp =
	      (Adp(i, XX) + a[1] * (Adp(i, YY) - b[1])) / (1 + pow (a[1], 2));
	    yp = a[1] * xp + b[1];

	    //---distance entre Pi et sa projection sur PjPj+1
	    //---distance between Pi and its projection onto PjPj+1
	    //la coordonn©e de projection xp fait partie du bout de droite PjPj+1
	    //the coordinated xp of the projected point belongs to the PjPj+1 segment
	    if (xp >= MIN (Bdp(j, XX), Bdp(j + 1, XX))
		&& xp <= MAX (Bdp(j, XX), Bdp(j + 1, XX)))
	      distance =
		sqrt (pow (Adp(i, XX) - xp, 2) + pow (Adp(i, YY) - yp, 2));
	    else
	      distance = -1;

	    //abscisse de la projection de Pi+1 sur la droite a1*x+b1
	    //x value of the projection of Pi+1 onto the line a1*x+b1
	    xp =
	      (Adp(i + 1, XX) + a[1] * (Adp(i + 1, YY) - b[1])) / (1 +
								 pow (a[1],
								      2));
	    yp = a[1] * xp + b[1];

	    if (xp >= MIN (Bdp(j, XX), Bdp(j + 1, XX))
		&& xp <= MAX (Bdp(j, XX), Bdp(j + 1, XX)))
	      {
		//              if (distance == -1)
		if (doubleequal (distance, (double) (-1)) == 1)
		  distance =
		    sqrt (pow (Adp(i + 1, XX) - xp, 2) +
			  pow (Adp(i + 1, YY) - yp, 2));
		else
		  distance =
		    MIN ((double) distance,
			 sqrt (pow (Adp(i + 1, XX) - xp, 2) +
			       pow (Adp(i + 1, YY) - yp, 2)));
	      }
	  }

	//---droite PjPj+1 verticale
	//---PjPj+1 line is vertical
	else
	  {
	    xp = Bdp(j, XX);
	    yp = Adp(i, YY);

	    if (yp >= MIN (Bdp(j, YY), Bdp(j + 1, YY))
		&& yp <= MAX (Bdp(j, YY), Bdp(j + 1, YY)))
	      distance =
		sqrt (pow (Adp(i, XX) - xp, 2) + pow (Adp(i, YY) - yp, 2));
	    else
	      distance = -1;

	    yp = Adp(i + 1, YY);
	    if (yp >= MIN (Bdp(j, YY), Bdp(j + 1, YY))
		&& yp <= MAX (Bdp(j, YY), Bdp(j + 1, YY)))
	      {
		//              if (distance == -1)
		if (doubleequal (distance, (double) (-1)) == 1)
		  distance =
		    sqrt (pow (Adp(i + 1, XX) - xp, 2) +
			  pow (Adp(i + 1, YY) - yp, 2));
		else
		  distance =
		    MIN ((double) distance,
			 sqrt (pow (Adp(i + 1, XX) - xp, 2) +
			       pow (Adp(i + 1, YY) - yp, 2)));

	      }
	  }

	//---PiPi+1 non verticale
	//---the PiPi+1 line is not vertical
	//      if (Adp(i + 1, XX) != Adp(i, XX))
	if (doubleequal (Adp(i + 1, XX), Adp(i, XX)) == 0)
	  {
	    //coefficients de la droite PiPi+1 d'©quation a0*x+b0
	    //coefficients of the PiPi+1 line. equation: a0*x+b0
	    a[0] = (Adp(i + 1, YY) - Adp(i, YY)) / (Adp(i + 1, XX) - Adp(i, XX));
	    b[0] =
	      (Adp(i, YY) * Adp(i + 1, XX) -
	       Adp(i + 1, YY) * Adp(i, XX)) / (Adp(i + 1, XX) - Adp(i, XX));

	    xp =
	      (Bdp(j, XX) + a[0] * (Bdp(j, YY) - b[0])) / (1 + pow (a[0], 2));
	    yp = a[0] * xp + b[0];
	    if (xp >= MIN (Adp(i, XX), Adp(i + 1, XX))
		&& xp <= MAX (Adp(i, XX), Adp(i + 1, XX)))
	      {
		if (doubleequal (distance, (double) (-1)) == 1)
		  distance =
		    sqrt (pow (Bdp(j, XX) - xp, 2) + pow (Bdp(j, YY) - yp, 2));
		else
		  distance =
		    MIN ((double) distance,
			 sqrt (pow (Bdp(j, XX) - xp, 2) +
			       pow (Bdp(j, YY) - yp, 2)));
	      }

	    xp =
	      (Bdp(j + 1, XX) + a[0] * (Bdp(j + 1, YY) - b[0])) / (1 +
								 pow (a[0],
								      2));
	    yp = a[0] * xp + b[0];
	    if (xp >= MIN (Adp(i, XX), Adp(i + 1, XX))
		&& xp <= MAX (Adp(i, XX), Adp(i + 1, XX)))
	      {
		//              if (distance == -1)
		if (doubleequal (distance, (double) (-1)) == 1)
		  distance =
		    sqrt (pow (Bdp(j + 1, XX) - xp, 2) +
			  pow (Bdp(j + 1, YY) - yp, 2));
		else
		  distance =
		    MIN ((double) distance,
			 sqrt (pow (Bdp(j + 1, XX) - xp, 2) +
			       pow (Bdp(j + 1, YY) - yp, 2)));
	      }
	  }

	//---PiPi+1 verticale
	//---the PiPi+1 line is vertical
	else
	  {
	    xp = Adp(i, XX);
	    yp = Bdp(j, YY);

	    if (yp >= MIN (Adp(i, YY), Adp(i + 1, YY))
		&& yp <= MAX (Adp(i, YY), Adp(i + 1, YY)))
	      distance =
		sqrt (pow (Bdp(j, XX) - xp, 2) + pow (Bdp(j, YY) - yp, 2));
	    else
	      distance = -1;

	    yp = Bdp(j + 1, YY);
	    if (yp >= MIN (Adp(i, YY), Adp(i + 1, YY))
		&& yp <= MAX (Adp(i, YY), Adp(i + 1, YY)))
	      {
		//              if (distance == -1)
		if (doubleequal (distance, (double) (-1)) == 1)
		  distance =
		    sqrt (pow (Bdp(j + 1, XX) - xp, 2) +
			  pow (Bdp(j + 1, YY) - yp, 2));
		else
		  distance =
		    MIN ((double) distance,
			 sqrt (pow (Bdp(j + 1, XX) - xp, 2) +
			       pow (Bdp(j + 1, YY) - yp, 2)));
	      }
	  }

	//---cas ou aucune projection ne fait partie d'un bout de droite.
	//---dans ce cas-la, on recherche le minimum des PiPj
	//---none of the projection belongs to a segment.
	//---therefore, we look for the minimum of the PiPj

	//      if (distance == -1)
	if (doubleequal (distance, (double) (-1)) == 1)
	  {
	    distance =
	      sqrt (pow (Bdp(j, XX) - Adp(i, XX), 2) +
		    pow (Bdp(j, YY) - Adp(i, YY), 2));

	    distance =
	      MIN ((double) distance,
		   sqrt (pow (Bdp(j + 1, XX) - Adp(i, XX), 2) +
			 pow (Bdp(j + 1, YY) - Adp(i, YY), 2)));

	    distance =
	      MIN ((double) distance,
		   sqrt (pow (Bdp(j + 1, XX) - Adp(i + 1, XX), 2) +
			 pow (Bdp(j + 1, YY) - Adp(i + 1, YY), 2)));

	    distance =
	      MIN ((double) distance,
		   sqrt (pow (Bdp(j, XX) - Adp(i + 1, XX), 2) +
			 pow (Bdp(j, YY) - Adp(i + 1, YY), 2)));
	  }


	//	printf("distance %g\n", distance);

	if (i == 0 && j == 0)
	  *distanceMinimale = distance;
	else
	  *distanceMinimale = MIN (distance, *distanceMinimale);
      }				// fin for j

  //  printf("DIST= %g\n", *distanceMinimale );

  return ;
}				// fin calculDistance

